@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: ioloop.py 1207 2013-04-08 01:18:24Z g.rodola $
+# $Id: ioloop.py 1242 2013-12-30 11:33:10Z g.rodola $
 
 #  ======================================================================
 #  Copyright (C) 2007-2013 Giampaolo Rodola' <g.rodola@gmail.com>
@@ -553,6 +553,10 @@ if hasattr(select, 'epoll'):
         _ERROR = select.EPOLLERR | select.EPOLLHUP
         _poller = select.epoll
 
+        def fileno(self):
+            """Return epoll() fd."""
+            return self._poller.fileno()
+
         def close(self):
             _IOLoop.close(self)
             self._poller.close()
@@ -572,6 +576,10 @@ if hasattr(select, 'kqueue'):
             self._kqueue = select.kqueue()
             self._active = {}
 
+        def fileno(self):
+            """Return kqueue() fd."""
+            return self._kqueue.fileno()
+
         def close(self):
             _IOLoop.close(self)
             self._kqueue.close()
@@ -677,6 +685,7 @@ else:                            # select() - POSIX and Windows
 
 _DISCONNECTED = frozenset((errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN,
                            errno.ECONNABORTED, errno.EPIPE, errno.EBADF))
+_RETRY = frozenset((errno.EAGAIN, errno.EWOULDBLOCK))
 
 class Acceptor(asyncore.dispatcher):
     """Same as base asyncore.dispatcher and supposed to be used to
@@ -845,7 +854,7 @@ class AsyncChat(asynchat.async_chat):
             return self.socket.send(data)
         except socket.error:
             why = sys.exc_info()[1]
-            if why.args[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
+            if why.args[0] in _RETRY:
                 return 0
             elif why.args[0] in _DISCONNECTED:
                 self.handle_close()
@@ -856,13 +865,6 @@ class AsyncChat(asynchat.async_chat):
     def recv(self, buffer_size):
         try:
             data = self.socket.recv(buffer_size)
-            if not data:
-                # a closed connection is indicated by signaling
-                # a read condition, and having recv() return 0.
-                self.handle_close()
-                return b('')
-            else:
-                return data
         except socket.error:
             why = sys.exc_info()[1]
             if why.args[0] in _DISCONNECTED:
@@ -870,6 +872,14 @@ class AsyncChat(asynchat.async_chat):
                 return b('')
             else:
                 raise
+        else:
+            if not data:
+                # a closed connection is indicated by signaling
+                # a read condition, and having recv() return 0.
+                self.handle_close()
+                return b('')
+            else:
+                return data
 
     def initiate_send(self):
         asynchat.async_chat.initiate_send(self)
