@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: bench.py 1171 2013-02-19 10:13:09Z g.rodola $
+# $Id: bench.py 1242 2013-12-30 11:33:10Z g.rodola $
 #
 #  pyftpdlib is released under the MIT license, reproduced below:
 #  ======================================================================
@@ -35,6 +35,8 @@ FTP server benchmark script.
 
 In order to run this you must have a listening FTP server with a user
 with writing permissions configured.
+This is a stand-alone script which does not depend from pyftpdlib.
+It just requires python >= 2.5.
 
 Example usages:
   python bench.py -u USER -p PASSWORD
@@ -48,45 +50,45 @@ Example usages:
 """
 
 # Some benchmarks (Linux 3.0.0, Intel core duo - 3.1 Ghz).
+
+# pyftpdlib 1.0.0:
 #
-# pyftpdlib 0.6.0:
+#   (starting with 6.7M of memory being used)
+#   STOR (client -> server)                              557.97 MB/sec  6.7M
+#   RETR (server -> client)                             1613.82 MB/sec  6.8M
+#   300 concurrent clients (connect, login)                1.20 secs    8.8M
+#   STOR (1 file with 300 idle clients)                  567.52 MB/sec  8.8M
+#   RETR (1 file with 300 idle clients)                 1561.41 MB/sec  8.8M
+#   300 concurrent clients (RETR 10.0M file)               3.26 secs    10.8M
+#   300 concurrent clients (STOR 10.0M file)               8.46 secs    12.6M
+#   300 concurrent clients (QUIT)                          0.07 secs
 #
-# (starting with 9M of RSS memory being used)
-# STOR (client -> server)                      528.02 MB/sec  9M RSS
-# RETR (server -> client)                      693.41 MB/sec  10M RSS
-# 200 concurrent clients (connect, login)        0.25 secs    10M RSS
-# 200 concurrent clients (RETR 10M file)         3.68 secs    11M RSS
-# 200 concurrent clients (STOR 10M file)         6.11 secs    11M RSS
-# 200 concurrent clients (quit)                  0.02 secs
 #
-# pyftpdlib 0.7.0:
-#  (starting with 6M of RSS memory being used)
-#   STOR (client -> server)                      512.80 MB/sec  9M RSS
-#   RETR (server -> client)                     1694.14 MB/sec  9M RSS
-#   200 concurrent clients (connect, login)        0.95 secs    11M RSS
-#   200 concurrent clients (RETR 10M file)         2.80 secs    12M RSS
-#   200 concurrent clients (STOR 10M file)         6.30 secs    13M RSS
-#   200 concurrent clients (quit)                  0.02 secs
+# proftpd 1.3.4a:
 #
-# proftpd 1.3.4rc2:
+#   (starting with 1.4M of memory being used)
+#   STOR (client -> server)                              554.67 MB/sec  3.2M
+#   RETR (server -> client)                             1517.12 MB/sec  3.2M
+#   300 concurrent clients (connect, login)                9.30 secs    568.6M
+#   STOR (1 file with 300 idle clients)                  484.11 MB/sec  570.6M
+#   RETR (1 file with 300 idle clients)                 1534.61 MB/sec  570.6M
+#   300 concurrent clients (RETR 10.0M file)               3.67 secs    568.6M
+#   300 concurrent clients (STOR 10.0M file)              11.21 secs    568.7M
+#   300 concurrent clients (QUIT)                          0.43 secs
 #
-#   (starting with 2M of RSS memory being used)
-#   STOR (client -> server)                      609.22 MB/sec  6M RSS
-#   RETR (server -> client)                     1313.77 MB/sec  6M RSS
-#   200 concurrent clients (connect, login)        7.53 secs    862M RSS
-#   200 concurrent clients (RETR 10M file)         2.76 secs    862M RSS
-#   200 concurrent clients (STOR 10M file)         6.39 secs    862M RSS
-#   200 concurrent clients (quit)                  0.23 secs
 #
 # vsftpd 2.3.2
 #
-#   (starting with 1M of RSS memory being used)
-#   STOR (client -> server)                      670.48 MB/sec  2M RSS
-#   RETR (server -> client)                     1505.18 MB/sec  2M RSS
-#   200 concurrent clients (connect, login)       12.61 secs    289M RSS
-#   200 concurrent clients (RETR 10M file)         2.30 secs    289M RSS
-#   200 concurrent clients (STOR 10M file)          N/A
-#   200 concurrent clients (quit)                  0.01
+#   (starting with 352.0K of memory being used)
+#   STOR (client -> server)                              607.23 MB/sec  816.0K
+#   RETR (server -> client)                             1506.59 MB/sec  816.0K
+#   300 concurrent clients (connect, login)               18.91 secs    140.9M
+#   STOR (1 file with 300 idle clients)                  618.99 MB/sec  141.4M
+#   RETR (1 file with 300 idle clients)                 1402.48 MB/sec  141.4M
+#   300 concurrent clients (RETR 10.0M file)               3.64 secs    140.9M
+#   300 concurrent clients (STOR 10.0M file)               9.74 secs    140.9M
+#   300 concurrent clients (QUIT)                          0.00 secs
+
 
 
 from __future__ import with_statement, division
@@ -116,11 +118,22 @@ PASSWORD = None
 TESTFN = "$testfile"
 BUFFER_LEN = 8192
 SERVER_PROC = None
+TIMEOUT = None
+PY3 = sys.version_info >= (3, 0)
 
+if PY3:
+    import builtins
+    print_ = getattr(builtins, "print")
 
-def print_(s):
-    sys.stdout.write(s + '\n')
-    sys.stdout.flush()
+    def b(s):
+        return s.encode("latin-1")
+else:
+    def print_(s):
+        sys.stdout.write(s + '\n')
+        sys.stdout.flush()
+
+    def b(s):
+        return s
 
 # http://goo.gl/6V8Rm
 def hilite(string, ok=True, bold=False):
@@ -147,7 +160,7 @@ def print_bench(what, value, unit=""):
                       hilite("%8.2f" % value),
                       unit)
     if server_memory:
-        s += "%s RSS" % hilite(server_memory.pop())
+        s += "%s" % hilite(server_memory.pop())
     print_(s.strip())
 
 # http://goo.gl/zeJZl
@@ -187,11 +200,27 @@ def human2bytes(s):
     return int(num * prefix[letter])
 
 def register_memory():
-    """Register RSS memory used by server process and its children."""
+    """Register an approximation of memory used by FTP server process
+    and all of its children.
+    """
+    # XXX How to get a reliable representation of memory being used is
+    # not clear. (rss - shared) seems kind of ok but we might also use
+    # the private working set via get_memory_maps().private*.
+    def get_mem(proc):
+        if os.name == 'posix':
+            mem = proc.get_ext_memory_info()
+            counter = mem.rss
+            if 'shared' in mem._fields:
+                counter -= mem.shared
+            return counter
+        else:
+            # TODO figure out what to do on Windows
+            return proc.get_memory_info().rss
+
     if SERVER_PROC is not None:
-        mem = SERVER_PROC.get_memory_info().rss
+        mem = get_mem(SERVER_PROC)
         for child in SERVER_PROC.get_children():
-            mem += child.get_memory_info().rss
+            mem += get_mem(child)
         server_memory.append(bytes2human(mem))
 
 def timethis(what):
@@ -217,8 +246,14 @@ def timethis(what):
 
 def connect():
     """Connect to FTP server, login and return an ftplib.FTP instance."""
-    ftp = ftplib.FTP()
-    ftp.connect(HOST, PORT)
+    if sys.version_info >= (2, 6):
+        # 'timeout' parameter is >= 2.6 only
+        ftp = ftplib.FTP(timeout=TIMEOUT)
+        ftp.connect(HOST, PORT)
+    else:
+        ftp = ftplib.FTP()
+        ftp.connect(HOST, PORT)
+        ftp.sock.settimeout(TIMEOUT)
     ftp.login(USER, PASSWORD)
     return ftp
 
@@ -241,7 +276,7 @@ def stor(ftp, size):
     """
     ftp.voidcmd('TYPE I')
     conn = ftp.transfercmd("STOR " + TESTFN)
-    chunk = 'x' * BUFFER_LEN
+    chunk = b('x') * BUFFER_LEN
     total_sent = 0
     while 1:
         sent = conn.send(chunk)
@@ -283,7 +318,7 @@ def bytes_per_second(ftp, retr=True):
         ftp.voidcmd('TYPE I')
         conn = ftp.transfercmd("STOR " + TESTFN)
         register_memory()
-        chunk = 'x' * BUFFER_LEN
+        chunk = b('x') * BUFFER_LEN
         stop_at = time.time() + 1
         while stop_at > time.time():
             bytes += conn.send(chunk)
@@ -328,11 +363,11 @@ class AsyncWriter(asynchat.async_chat):
         def __init__(self, size):
             self.size = size
             self.sent = 0
-            self.chunk = 'x' * BUFFER_LEN
+            self.chunk = b('x') * BUFFER_LEN
 
         def more(self):
             if self.sent >= self.size:
-                return ''
+                return b('')
             self.sent += len(self.chunk)
             return self.chunk
 
@@ -349,8 +384,8 @@ class AsyncQuit(asynchat.async_chat):
     def __init__(self, sock):
         asynchat.async_chat.__init__(self, sock)
         self.in_buffer = []
-        self.set_terminator('\r\n')
-        self.push('QUIT\r\n')
+        self.set_terminator(b('\r\n'))
+        self.push(b('QUIT\r\n'))
 
     def collect_incoming_data(self, data):
         self.in_buffer.append(data)
@@ -376,7 +411,7 @@ class OptFormatter(optparse.IndentedHelpFormatter):
         return ''.join(result)
 
 def main():
-    global HOST, PORT, USER, PASSWORD, SERVER_PROC
+    global HOST, PORT, USER, PASSWORD, SERVER_PROC, TIMEOUT
     USAGE = "%s -u USERNAME -p PASSWORD [-H] [-P] [-b] [-n] [-s] [-k]" % __file__
     parser = optparse.OptionParser(usage=USAGE,
                                    epilog=__doc__[__doc__.find('Example'):],
@@ -384,7 +419,8 @@ def main():
     parser.add_option('-u', '--user', dest='user', help='username')
     parser.add_option('-p', '--pass', dest='password', help='password')
     parser.add_option('-H', '--host', dest='host', default=HOST, help='hostname')
-    parser.add_option('-P', '--port', dest='port', default=PORT, help='port')
+    parser.add_option('-P', '--port', dest='port', default=PORT, help='port',
+                      type=int)
     parser.add_option('-b', '--benchmark', dest='benchmark', default='transfer',
                       help="benchmark type ('transfer', 'concurrence', 'all')")
     parser.add_option('-n', '--clients', dest='clients', default=200, type="int",
@@ -395,6 +431,8 @@ def main():
                            "(e.g. '10M')")
     parser.add_option('-k', '--pid', dest='pid', default=None, type="int",
                       help="the PID of the server process to bench memory usage")
+    parser.add_option('-t', '--timeout', dest='timeout',
+                      default=TIMEOUT, type="int", help="the socket timeout")
 
 
     options, args = parser.parse_args()
@@ -405,6 +443,7 @@ def main():
         PASSWORD = options.password
         HOST = options.host
         PORT = options.port
+        TIMEOUT = options.timeout
         try:
             FILE_SIZE = human2bytes(options.filesize)
         except (ValueError, AssertionError):
@@ -490,7 +529,7 @@ def main():
     # start benchmark
     if SERVER_PROC is not None:
         register_memory()
-        print_("(starting with %s of RSS memory being used)" \
+        print_("(starting with %s of memory being used)" \
                % hilite(server_memory.pop()))
     if options.benchmark == 'transfer':
         bench_stor()
