@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: test_contrib.py 1188 2013-03-05 23:15:13Z g.rodola $
+# $Id: test_contrib.py 1242 2013-12-30 11:33:10Z g.rodola $
 
 #  pyftpdlib is released under the MIT license, reproduced below:
 #  ========================================================================
@@ -172,6 +172,7 @@ class TestCallbacksThreadMixin(ThreadFTPTestMixin, TestCallbacks): pass
 class TestIPv4EnvironmentThreadMixin(ThreadFTPTestMixin, TestIPv4Environment): pass
 class TestIPv6EnvironmentThreadMixin(ThreadFTPTestMixin, TestIPv6Environment): pass
 class TestCornerCasesThreadMixin(ThreadFTPTestMixin, TestCornerCases): pass
+class TestFTPServerThreadMixin(ThreadFTPTestMixin, TestFTPServer): pass
 
 
 # =====================================================================
@@ -207,6 +208,7 @@ class TestFtpAbortMProcMixin(MProcFTPTestMixin, TestFtpAbort): pass
 class TestIPv4EnvironmentMProcMixin(MProcFTPTestMixin, TestIPv4Environment): pass
 class TestIPv6EnvironmentMProcMixin(MProcFTPTestMixin, TestIPv6Environment): pass
 class TestCornerCasesMProcMixin(MProcFTPTestMixin, TestCornerCases): pass
+class TestFTPServerMProcMixin(MProcFTPTestMixin, TestFTPServer): pass
 
 
 # =====================================================================
@@ -776,6 +778,7 @@ def test_main():
         #TestConfigurableOptionsThreadMixin,
         TestCallbacksThreadMixin,
         TestCornerCasesThreadMixin,
+        TestFTPServerThreadMixin,
     ]
     tests += ftp_thread_tests
 
@@ -794,6 +797,7 @@ def test_main():
             #TestConfigurableOptionsMProcMixin,
             #TestCallbacksMProcMixin,
             TestCornerCasesMProcMixin,
+            TestFTPServerMProcMixin,
         ]
         tests += ftp_mproc_tests
 
